using ldap_cesi.DTOs.Inputs.Service;
using ldap_cesi.Services.Interfaces;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace ldap_cesi.Controllers;

[ApiController]
[Route("api/service")]
public class ServicesController : ControllerBase
{
    private IServiceService _serviceService;

    public ServicesController(IServiceService serviceService)
    {
        _serviceService = serviceService;
    }
    
    // GET: api/services
    /// <summary>
    /// Endpoint qui retournes tous les services
    /// </summary>
    /// <returns>Retourne tous les services</returns>
    [HttpGet]
    // [Authorize(Roles = "admin")]
    public async Task<ActionResult> GetServices()
    {
        var result = await _serviceService.GetAll();
        return result.Success ? Ok(result) : BadRequest(result);
    }            
    
    // GET: api/services
    /// <summary>
    /// Endpoint retourne le service correspondant à l'id en param
    /// </summary>
    /// <returns>Service</returns>
    [HttpGet("{id}")]
    public async Task<ActionResult> GetServices(int id)
    {
        var result = await _serviceService.GetById(id);
        return result.Success ? Ok(result) : BadRequest(result);
    }        
    
    // {POST}: api/services
    /// <summary>
    /// Endpoint qui créé un service
    /// </summary>
    /// <returns>Retourne tous les services</returns>
    [HttpPost]
    public async Task<ActionResult> CreateService([FromBody] ServiceCreateDto serviceInputDto)
    {
        var result = await _serviceService.CreateService(serviceInputDto);
        return result.Success ? Ok(result) : BadRequest(result);
    }    
}