using AutoMapper;
using ldap_cesi.DTOs.Inputs.Service;
using ldap_cesi.Entities;
using ldap_cesi.Models;
using ldap_cesi.Repository.Services;
using ldap_cesi.Services.Interfaces;

namespace ldap_cesi.Services;

public class ServiceService : IServiceService
{
    private readonly IRepositoryService _repositoryService;
    private readonly IMapper _mapper;

    public ServiceService(IRepositoryService repositoryService, IMapper mapper)
    {
        _repositoryService = repositoryService;
        _mapper = mapper;
    }


    public async Task<IResponseDataModel<List<Service>>> GetAll()
    {
        var services = await _repositoryService.GetAllAsync();
        return new ResponseDataModel<List<Service>>
        {
            Success = true,
            Data = services,
            StatusCode = 200,
        };
    }    
    
    public async Task<IResponseDataModel<Service>> GetById(int id)
    {
        var service = await _repositoryService.GetByIdAsync(id);
        return new ResponseDataModel<Service>
        {
            Success = true,
            Data = service,
            StatusCode = 200,
        };
    }

    public async Task<IResponseDataModel<string>> CreateService(ServiceCreateDto serviceCreateDto)
    {
        var service = _mapper.Map<Service>(serviceCreateDto);
        var response = await _repositoryService.AddAsync(service);
        return new ResponseDataModel<string>
        {
            Success = true,
            Data = response.Id.ToString(),
        };
    }
}