using AutoMapper;
using ldap_cesi.DTOs.Inputs;
using ldap_cesi.DTOs.Outputs.Utilisateur;
using ldap_cesi.Entities;
using ldap_cesi.Models;
using ldap_cesi.Repository.Services;
using ldap_cesi.Services.Interfaces;

namespace ldap_cesi.Services;

public class UtilisateurService : IUtilisateurService
{
    private readonly IRepositoryUtilisateur _repositoryUtilisateur;
    private readonly IJwtService _jwtService;
    private readonly IMapper _mapper;
    public UtilisateurService(IRepositoryUtilisateur repositoryUtilisateur, IJwtService jwtService, IMapper mapper)
    {
        _repositoryUtilisateur = repositoryUtilisateur;
        _jwtService = jwtService;
        _mapper = mapper;
    }

    public async Task<IResponseDataModel<List<Utilisateur>>> GetAll()
    {
        var utilisateurs = await _repositoryUtilisateur.GetAllAsync();
        return new ResponseDataModel<List<Utilisateur>>
        {
            Success = true,
            StatusCode = 200,
            Data = utilisateurs,
        };
    }

    public async Task<IResponseDataModel<Utilisateur>> GetById(int id)
    {
        var utililisateur = await _repositoryUtilisateur.GetByIdAsync(id);
        return new ResponseDataModel<Utilisateur>
        {
            Success = true,
            StatusCode = 200,
            Data = utililisateur,
        };
    }
    
    public async Task<IResponseDataModel<UtilisateurOutputDto>> Login(UtilisateurLoginDto utilisateurInput)
    {
        var utilisateur = await _repositoryUtilisateur.GetByEmailAsync(utilisateurInput.Email);

        if (utilisateur == null)
        {
            return new ResponseDataModel<UtilisateurOutputDto>
            {
                Success = false,
                StatusCode = 404,
                Message = "Utilisateur non trouvé."
            };
        }
        
        if (!BCrypt.Net.BCrypt.Verify(utilisateurInput.MotDePasse, utilisateur.MotDePasse))
        {
            return new ResponseDataModel<UtilisateurOutputDto>
            {
                Success = false,
                StatusCode = 401,
                Message = "Mot de passe incorrect."
            };
        }
        var token = _jwtService.GenerateToken(utilisateur);
        var utilisateurOutputDto = _mapper.Map<UtilisateurOutputDto>(utilisateur);
        
        return new ResponseDataModel<UtilisateurOutputDto>
        {
            Success = true,
            StatusCode = 200,
            Data = utilisateurOutputDto,
            Token = token,
            Message = "Connexion réussie."
        };
    }
}